-------------------------------------------------------------------------------
--! @file       cloc_datapath_encdec.vhd
--! @author     Farnoud Farahmand
--! @brief      Encryption Decryption unit for CLOC 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.AES_pkg.all;


entity encdec is
generic ( n: integer:= 128);
--!inputs for encryption and decryption 
    Port ( 
		   rst        : in std_logic;
		   clk        : in std_logic;
			v          : in  STD_LOGIC_vector(n-1 downto 0);
         bdi        : in  STD_LOGIC_vector (n-1 downto 0);
			key        : in  STD_LOGIC_vector (n-1 downto 0);
         bdo        : out STD_LOGIC_vector (n-1 downto 0);
			c_out      : out STD_LOGIC_vector (n-1 downto 0);
			sel_ed1    : in std_logic;
         sel_ed3    : in std_logic;
			sel_bdo	  : in std_logic;
			init       : in std_logic;
			start      : in std_logic;
			bdi_valid_bytes : in STD_LOGIC_vector (15 downto 0); 
			bdi_pad_loc	    : in STD_LOGIC_vector (15 downto 0)
		 );
end encdec;

architecture dataflow of encdec is

--! selector signals
signal muxa_1_i      : std_logic_vector (127 downto 0);
signal muxa_op       : std_logic_vector (127 downto 0);
signal aes_in        : std_logic_vector (127 downto 0);
signal aes_op2        : std_logic_vector (127 downto 0);
signal xora_op       : std_logic_vector (127 downto 0);
signal muxb_0_i      : std_logic_vector (127 downto 0);
signal muxb_op       : std_logic_vector (127 downto 0);
signal msb_i         : std_logic_vector (127 downto 0);
signal msb_o         : std_logic_vector (127 downto 0);
signal muxc_0_i      : std_logic_vector (127 downto 0);
signal muxc_1_i      : std_logic_vector (127 downto 0);
signal muxc_op       : std_logic_vector (127 downto 0);

signal out_sel        : std_logic;
signal pad0_out       : std_logic_vector (127 downto 0);
signal mux_pad0_out   : std_logic_vector (127 downto 0);
signal pad1_out       : std_logic_vector (127 downto 0);
signal mux_pad1_out   : std_logic_vector (127 downto 0);

signal valid_bytes_sig   : std_logic_vector (127 downto 0);
signal pad_loc_sig       : std_logic_vector (127 downto 0);
signal aes_op2_and_padbl : std_logic_vector (127 downto 0);

begin

-- top part implementation

u1 : entity work.fix1(dataflow)
port map( fix1_i => muxc_op,
          fix1_o => muxa_1_i);
			 
muxa_op <= v when sel_ed1='0' else  muxa_1_i;
         
aes_in  <= muxa_op;
-- aes enc block
u2 : entity work.AES_Enc (structure)
port map ( din    => aes_in,
           dout   => aes_op2,
			  clk    => clk,
			  key    => key,
			  init   => init,
			  start  => start,
			  rst    => rst
			  );
-- end of aes enc bloc

valid_bytes : entity work.bit_expand (structure)
port map ( input     => bdi_valid_bytes,
		   output    => valid_bytes_sig
			  );	
aes_op2_and_padbl <= aes_op2 and valid_bytes_sig;	

mux_pad0_out <= aes_op2 when out_sel ='0' else aes_op2_and_padbl;           
			  
xora_op <= bdi xor mux_pad0_out;
--selecting input bdi for decryption algorithm
muxc_op <= xora_op when sel_ed3 ='1' else bdi;           		  
			  
out_sel  <= '0' when bdi_pad_loc = x"0000" else '1' ;

bdo       <=  xora_op when sel_bdo ='0' else v;
c_out     <=  xora_op;

end dataflow;

