-------------------------------------------------------------------------------
--! @file       cloc_datapath_ph.vhd
--! @author     Farnoud Farahmand
--! @brief      PRF and HASH unit for CLOC 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.AES_pkg.all;


entity ph is
generic (n : integer := 128);
    Port (    bdi          : in std_logic_vector (n-1 downto 0);
			  key          : in std_logic_vector (n-1 downto 0);
			  c_in         : in  STD_LOGIC_vector (n-1 downto 0);
			  init         : in std_logic;
			  start        : in std_logic;
			  
			  v            : out std_logic_vector (127 downto 0); 
			  done         : out std_logic;
			  done_init    : out std_logic;
			  tag_match    : out std_logic;
			  msb_a        : out std_logic;

			  sel_ph1      : in std_logic;
			  sel_ph2      : in std_logic;
			  sel_ph3      : in std_logic;
			  sel_ph4      : in std_logic;
			  sel_ph5      : in std_logic;
			  sel_ph6      : in std_logic_vector (1 downto 0);
			  sel_ph7      : in std_logic;
			  
			  sel_muxff    : in std_logic;
			  sel_ph_last  : in std_logic_vector (1 downto 0);
			  
           en_npub_reg  : in std_logic;
			  clk          : in  STD_LOGIC;
			  rst          : in std_logic
			  );
end ph;

architecture dataflow of ph is

signal muxa_0_i : std_logic_vector (127 downto 0);
signal muxa_1_i : std_logic_vector (127 downto 0);
signal muxa_op  : std_logic_vector (127 downto 0);

signal muxb_0_i : std_logic_vector (127 downto 0);
signal muxb_1_i : std_logic_vector (127 downto 0);
signal muxb_op  : std_logic_vector (127 downto 0);

signal muxc_0_i : std_logic_vector (127 downto 0);
signal muxc_1_i : std_logic_vector (127 downto 0);
signal muxc_op  : std_logic_vector (127 downto 0);

signal muxd_0_i : std_logic_vector (127 downto 0);
signal muxd_1_i : std_logic_vector (127 downto 0);
signal muxd_op  : std_logic_vector (127 downto 0);

signal aes_in   : std_logic_vector (127 downto 0);
signal aes_op   : std_logic_vector (127 downto 0);

signal muxe_0_i : std_logic_vector (127 downto 0);
signal muxe_1_i : std_logic_vector (127 downto 0);
signal muxe_op  : std_logic_vector (127 downto 0);

signal muxf_0_i : std_logic_vector (127 downto 0);
signal muxf_1_i : std_logic_vector (127 downto 0);
signal muxf_op  : std_logic_vector (127 downto 0);

signal muxg_0_i : std_logic_vector (127 downto 0);
signal muxg_1_i : std_logic_vector (127 downto 0);
signal muxg_op  : std_logic_vector (127 downto 0);

signal muxh_0_i : std_logic_vector (127 downto 0);
signal muxh_1_i : std_logic_vector (127 downto 0);
signal muxh_op  : std_logic_vector (127 downto 0);

signal muxff_out : std_logic_vector (127 downto 0);

signal vs        : std_logic_vector (127 downto 0);

signal npub      : std_logic_vector (127 downto 0);
--signal length_pad     : std_logic_vector (3 downto 0);



signal muxb_0_i_p        : std_logic_vector (127 downto 0);
signal muxb_0_if1        : std_logic_vector (127 downto 0);
signal muxb_1_if2        : std_logic_vector (127 downto 0);
     

begin

muxff_out <= bdi when sel_muxff ='0' else c_in;

--! getting input from mux of top_level			  
u1 : entity work.fix0 (dataflow)
port map ( fix0_i => muxff_out, -- for first roud a(1)
           fix0_o => muxb_1_i);
			  
--xoring previous calculated value with present value for round 2 onwards
muxb_0_i <= muxff_out xor muxg_op;


u9 : entity work.f1(dataflow)			  
port map ( inf1 => muxb_0_i,
           opf1 => muxb_0_if1
			  );
			  
u10 : entity work.f2 (dataflow)
port map ( inf2 => muxb_0_i,
           opf2 => muxb_1_if2
			  );

with sel_ph_last select			  
muxb_0_i_p <= muxb_0_i   when "00",
              muxb_0_if1 when "01" ,
			  muxb_1_if2 when others;




-- selection for the first round or the subsequent round
muxd_0_i <= muxb_0_i_p when sel_ph1 ='0' else  muxb_1_i;
          
--implementation for tag function
-- passing the value through g1 function if cipher is 0 			  
u2 : entity work.g1 (dataflow)
port map ( inpg1 => vs, -- this is the input v
           opg1 => muxc_0_i);
			  
-- passing the value through g2 if we have cipher generated from encryption function
u3 : entity work.g2 (dataflow)
port map ( inpg2 => vs, -- this is the input generated from after excution of hash function
           opg2 => muxc_1_i);
			  
-- selection between g1 and g2
muxd_1_i <= muxc_0_i when sel_ph2 ='0' else muxc_1_i;
            
				
--selection between hash function and tag function			  
muxd_op <= muxd_0_i when sel_ph3 = '0' else muxd_1_i;
           

--implementing aes enc block
u4 : entity work.AES_Enc(structure)
port map ( din    => muxd_op,
           dout   => aes_op,
			  key    => key,
			  init   => init,
			  start  => start,
			  done   => done,
			  done_init  => done_init,
			  --en => en,
			  clk    => clk,
			  rst    => rst
			  );
-- end of, aes block

muxe_1_i <= aes_op;
--pass first AD through tweak function 'h' 
u5 : entity work.h(dataflow)
port map ( hin => aes_op,
           hop => muxe_0_i
			  );
			  
-- selection between tweaked value and normal value
muxe_op <= muxe_0_i when sel_ph4 ='0' else muxe_1_i;
           
--Registering npub

Npub_Reg : entity work.Register_s (Reg_arc)
port map ( d   => bdi,
           q   => npub,
		   enable  => en_npub_reg,
		   reset => rst,
		   clock => clk
			  ); 

		   
--adding nonce word received from user for the last round
muxf_0_i <= muxe_op xor npub;
muxf_1_i <= muxe_op;

-- selection between nonce added word and normal word
muxf_op <= muxf_0_i when sel_ph5 ='0' else muxf_1_i ;
           
			  
-- implementation for tweak functions f1 and f2
u6 : entity work.f1(dataflow)			  
port map ( inf1 => muxf_op,
           opf1 => muxg_0_i
			  );
			  
u7 : entity work.f2 (dataflow)
port map ( inf2 => muxf_op,
           opf2 => muxg_1_i
			  );
			  
with sel_ph6 select			  
muxg_op <= muxg_0_i when "00",
           muxg_1_i when "01" ,
			  muxf_op when others;
			  
-- passing the last block of tag function through MSB function			  
u8 : entity work.msb (dataflow)
port map ( msb_in => muxg_op,
           msb_op => muxh_0_i
			  );
     
	  
v   <=  muxg_op;
vs  <=  muxg_op;
			  
-- TAG Compare
       tag_match <= '1' when bdi = muxg_op else '0' ; 

        msb_a    <= bdi(127);		  
		
end dataflow;

