-------------------------------------------------------------------------------
--! @file       cloc_datapath_top_level.vhd
--! @author     Farnoud Farahmand
--! @brief      Top level Datapath unit for CLOC 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity top_level_datapath is
generic (n : integer := 128);
port ( 
         bdi     : in std_logic_vector (n-1 downto 0);
         bdo     : out std_logic_vector (n-1 downto 0);
		 key     : in std_logic_vector (n-1 downto 0);
		 rst     : in std_logic;
		 clk     : in std_logic;
		 init    : in std_logic;
		 start   : in std_logic;
		 sel_ph1 : in std_logic;
		 sel_ph2 : in std_logic;
		 sel_ph3 : in std_logic;
		 sel_ph4 : in std_logic;
		 sel_ph5 : in std_logic;
		 sel_ph6 : in std_logic_vector (1 downto 0);
		 sel_ph7 : in std_logic;
		 sel_ed1 : in std_logic;
		 sel_ph_last  : in std_logic_vector (1 downto 0);
		 sel_bdo	  : in std_logic;
		 en_npub_reg : in std_logic;
  		  init_enc_dec : in std_logic;
		  done         : out std_logic;
		  done_init    : out std_logic;
		  tag_match    : out std_logic;
		  msb_a        : out std_logic;
		  bdi_valid_bytes : in STD_LOGIC_vector (15 downto 0); 
		  bdi_pad_loc	    : in STD_LOGIC_vector (15 downto 0)
		 );
		 
		 
end top_level_datapath;

architecture dataflow of top_level_datapath is

signal v            : std_logic_vector (127 downto 0);
signal c_in         : STD_LOGIC_vector (127 downto 0);

begin
			  
u1 : entity work.ph(dataflow)
port map ( 
			  rst   => rst,
			  clk   => clk,
			  bdi   => bdi ,
              v     => v,
			  init  => init,
			  start => start,
			  key   => key,
			  sel_ph1 => sel_ph1,
			  sel_ph2 => sel_ph2,
			  sel_ph3 => sel_ph3,
			  sel_ph4 => sel_ph4,
			  sel_ph5 => sel_ph5,
			  sel_ph6 => sel_ph6,
			  sel_ph7 => sel_ph7, 
			  en_npub_reg	=> en_npub_reg,
			  c_in    => c_in,
			  done_init   => done_init,
			  tag_match   => tag_match,
			  sel_muxff   => init_enc_dec,
			  sel_ph_last => sel_ph_last,
			  done        => done,
			  msb_a       => msb_a
			  );
			  



u2 : entity work.encdec(dataflow)
port map ( 
              bdi     => bdi,
              v       => v,
			  rst     => rst,
			  clk 	  => clk,
			  sel_ed1 => sel_ed1,
			  sel_ed3 => init_enc_dec,
			  sel_bdo => sel_bdo,
			  start   => start,
			  init    => init,
			  key     => key,
			  bdo     => bdo,
			  c_out   => c_in,
			  bdi_valid_bytes => bdi_valid_bytes,
			  bdi_pad_loc => bdi_pad_loc
			  );

end dataflow;

